/*
 * httpupdater.h
 *
 *  Created on: 08.06.2009
 *      Author: Chebotarev Roman
 */

#ifndef HTTPUPDATER_H_
#define HTTPUPDATER_H_

#include <QtDebug>
#include <QFile>
#include <QObject>
#include <QStringList>
#include <QHttp>
#include <QUrl>
#include <QList>
#include <QSslError>
#include <QSslSocket>

#define MSG(s)	qDebug() << s

class HttpUpdater : public QObject
{
	Q_OBJECT
public:
	HttpUpdater(QString host = "", quint16 port = 80, QString path = "", QString updateInfo = "",
			bool checkImmediate = false, bool httpsMode = false, bool denyInvalidCert = true);

	void 	setHost(QString host) { updateServer = host; }
	void	setPort(quint32 port) { updatePort = port; }
	void 	setPath(QString path) { updatePath = path; }
	void 	setUpdateInfoFilename(QString updateInfo) { updateInfoFilename = updateInfo; }
	void 	setHttpsMode(bool use) { useHttps = use; }
	void 	setDenyInvalidCert(bool deny) { denyInvalidCertificate = deny; }
	void 	setUser(QString login = "", QString passwd = "") { httpLogin = login; httpPasswd = passwd; }

	void	setUpdateFileLocalName(QString name) { updateFileLocalName = name; }
	void	downloadUpdate(QString updateUrl);

	bool 	isCheckFinished() { return checkFin; }
	bool 	isErrorOccured() { return updateError; }
	const QString & errorDescription() { return errorDescr; }
	quint32	binVersion() { return latestBinVersion; }
	const QString & strVersion() { return latestStrVersion; }
	const QString & updateUrl() { return updateFileUrl; }
	const QString & infoContent() { return updateInfoContent; }

	~HttpUpdater();

Q_SIGNALS:
	void		checkUpdateFinished();
	void		downloadUpdateFinished();

public Q_SLOTS:
	void	check();

private Q_SLOTS:
	void		checkFinished(bool error);
	void		downloadFinished(bool error);

private:
	bool	parseUpdateInfo(QString updateInfoData);

	enum	httpResponseCodes
	{
		httpOk			= 200,
		httpNotFound	= 404
	};
	QHttp		*httpObj;
	QTcpSocket	*httpSocket;
	QSslSocket	*httpSslSocket;
	bool		useHttps;
	bool		denyInvalidCertificate;
	QString		updateServer;
	quint32		updatePort;
	QString		updatePath;
	QString		updateInfoFilename;
	QString		httpLogin;
	QString		httpPasswd;
	bool		checkFin;
	bool		updateError;
	QString		errorDescr;
	QString		latestStrVersion;
	quint32		latestBinVersion;
	QString		updateFileUrl;
	QString		updateInfoContent;
	QString		updateFileLocalName;
	QFile		*localUpdateFile;
};

#endif /* HTTPUPDATER_H_ */
